<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\interfaces;

interface MasqueradeServiceInterface
{
    public function startMasquerade(CanBeMasqueradedInterface $to): void;

    public function endMasquerade(): void;

    public function isMasquerading(): bool;

    public function getMasqueraded(): CanBeMasqueradedInterface;

}