<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\enums;
enum Role: string
{
    case SUPER_ADMINISTRATOR = 'super-administrator';
    case ADMINISTRATOR = 'administrator';
    case STAFF = 'staff';
    case USER = 'user';

}